import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import powerEvaluators.BasicAttackEvaluator;
import powerEvaluators.CarefulAttackEvaluator;
import powerEvaluators.CarefulAttackEvaluator2;
import powerEvaluators.PowerEvaluator;
import powerEvaluators.PowerExpectedDamageEvaluator;
import powerEvaluators.ReapingStrike1HandedEvaluator;
import powerEvaluators.ReapingStrike2HandedEvaluator;
import powerEvaluators.TwinStrikeEvaluator;
import powerEvaluators.TwinStrikeEvaluator2;
import powerEvaluators.TwinStrikeEvaluator3;
import statAnalysis.OneOnOneExpectedDamageStatAnalysisProcessor;
import statAnalysis.StatAnalysisProcessor;
import statAnalysis.singleStatAnalysis.OneOnOneDamageBonusAbilityModDeltaSingleStatAnalysis;
import statAnalysis.singleStatAnalysis.OneOnOneExpectedDamageSingleStatAnalysis;
import statAnalysis.singleStatAnalysis.OneOnOneHitChanceDistributionSingleStatAnalysis;
import statAnalysis.singleStatAnalysis.SingleStatAnalysis;

/**
 * Processes a list of {@link StatAnalysisProcessor}s and their attendant
 * {@link SingleStatAnalysis} objects. Each {@link SingleStatAnalysis} provides
 * a report on a single statistic, and all reports are compiled into a single
 * file. The calculated expected damage results are written to a second file. 
 * 
 */
public class EvaluatePowers
{
  /**
   * Weapon damage dice.
   */
  private static int[] weaponDamageList = {4, 6, 8, 10};
  /**
   * Crit ranges.
   */
  private static int[] critRangeList = {20, 19, 18};
  /**
   * Rolls required to hit (includes all to hit bonuses).
   */
  private static int[] rollRequiredToHitList = {19, 16, 13, 11, 9, 6, 3};
  /**
   * Damage bonus to a weapon's attack, NOT including the relevant ability mod.
   */
  private static int[] bonusDamageList = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
  /**
   * Ability mods.
   */
  private static int[] abilityModBonusList = {0, 1, 2, 3, 4, 5, 6, 7, 8};
  /**
   * A list of {@link StatAnalysisProcessor} objects.
   */
  private static List<StatAnalysisProcessor> statAnalysisProcessors = new ArrayList<StatAnalysisProcessor>();

  /**
   * @param args
   *          The first argument is the name of the file which contains the
   *          expected damage results for each combination of weapon damage,
   *          crit range, chance to hit, bonus damage, and ability mod
   *          contribution to damage.
   * 
   * The second argument is the name of the file which contains each of the stat
   * analyses.
   */
  public static void main(String[] args)
  {
    BufferedWriter rawDataWriter = null;
    BufferedWriter statsAnalysisWriter = null;

    try
    {
      int maxDamage;
      int critRange;
      int rollRequiredToHit;
      int bonusDamage;
      int abilityModBonus;
      File rawDataOutputFile = new File(args[0]);
      File statsAnalysisOutputFile = new File(args[1]);
      Map<Class, PowerExpectedDamageEvaluator> evaluatorMap = new HashMap<Class, PowerExpectedDamageEvaluator>();
      List<PowerExpectedDamageEvaluator> evaluatorList = new ArrayList<PowerExpectedDamageEvaluator>();
      TwinStrikeEvaluator twinStrikeEvaluator = new TwinStrikeEvaluator();
      TwinStrikeEvaluator2 twinStrikeEvaluator2 = new TwinStrikeEvaluator2();
      TwinStrikeEvaluator3 twinStrikeEvaluator3 = new TwinStrikeEvaluator3();
      CarefulAttackEvaluator carefulStrikeEvaluator = new CarefulAttackEvaluator();
      CarefulAttackEvaluator2 carefulStrikeEvaluator2 = new CarefulAttackEvaluator2();
      BasicAttackEvaluator basicAttackEvaluator = new BasicAttackEvaluator();
      ReapingStrike1HandedEvaluator reapingStrike1HandedEvaluator = new ReapingStrike1HandedEvaluator();
      ReapingStrike2HandedEvaluator reapingStrike2HandedEvaluator = new ReapingStrike2HandedEvaluator();

      initStatProcessors();
      evaluatorMap.put(TwinStrikeEvaluator.class, twinStrikeEvaluator);
      evaluatorMap.put(twinStrikeEvaluator2.getClass(), twinStrikeEvaluator2);
      evaluatorMap.put(twinStrikeEvaluator3.getClass(), twinStrikeEvaluator3);
      evaluatorMap.put(carefulStrikeEvaluator.getClass(), carefulStrikeEvaluator);
      evaluatorMap.put(carefulStrikeEvaluator2.getClass(), carefulStrikeEvaluator2);
      evaluatorMap.put(basicAttackEvaluator.getClass(), basicAttackEvaluator);
      evaluatorMap.put(reapingStrike1HandedEvaluator.getClass(), reapingStrike1HandedEvaluator);
      evaluatorMap.put(reapingStrike2HandedEvaluator.getClass(), reapingStrike2HandedEvaluator);
      evaluatorList.addAll(evaluatorMap.values());
      rawDataWriter = new BufferedWriter(new FileWriter(rawDataOutputFile));
      statsAnalysisWriter = new BufferedWriter(new FileWriter(statsAnalysisOutputFile));
      for (int i = 0; i < weaponDamageList.length; i++)
      {
        maxDamage = weaponDamageList[i];
        for (int j = 0; j < critRangeList.length; j++)
        {
          critRange = critRangeList[j];
          for (int k = 0; k < rollRequiredToHitList.length; k++)
          {
            rollRequiredToHit = rollRequiredToHitList[k];
            for (int m = 0; m < bonusDamageList.length; m++)
            {
              bonusDamage = bonusDamageList[m];
              for (int n = 0; n < abilityModBonusList.length; n++)
              {
                abilityModBonus = abilityModBonusList[n];

                boolean markBest = false;

                rawDataWriter.write("Damage die: " + maxDamage);
                rawDataWriter.newLine();
                rawDataWriter.write("Critical range: " + critRange);
                rawDataWriter.newLine();
                rawDataWriter.write("d20 roll required to hit: " + rollRequiredToHit);
                rawDataWriter.newLine();
                rawDataWriter.write("Damage bonus (excluding ability score bonus): " + bonusDamage);
                rawDataWriter.newLine();
                rawDataWriter.write("Ability score bonus: " + abilityModBonus);
                rawDataWriter.newLine();
                twinStrikeEvaluator.evaluate(maxDamage, critRange, rollRequiredToHit, bonusDamage, abilityModBonus);
                twinStrikeEvaluator2.evaluate(maxDamage, critRange, rollRequiredToHit, bonusDamage, abilityModBonus);
                twinStrikeEvaluator3.evaluate(maxDamage, critRange, rollRequiredToHit, bonusDamage, abilityModBonus);
                carefulStrikeEvaluator.evaluate(maxDamage, critRange, rollRequiredToHit, bonusDamage, abilityModBonus);
                carefulStrikeEvaluator2.evaluate(maxDamage, critRange, rollRequiredToHit, bonusDamage, abilityModBonus);
                basicAttackEvaluator.evaluate(maxDamage, critRange, rollRequiredToHit, bonusDamage, abilityModBonus);
                reapingStrike1HandedEvaluator.evaluate(maxDamage, critRange, rollRequiredToHit, bonusDamage, abilityModBonus);
                reapingStrike2HandedEvaluator.evaluate(maxDamage, critRange, rollRequiredToHit, bonusDamage, abilityModBonus);

                Collections.sort(evaluatorList, Collections.reverseOrder());
                for (PowerEvaluator powerEvaluator : evaluatorList)
                {
                  if (!markBest)
                  {
                    rawDataWriter.write("***");
                    markBest = true;
                  }
                  rawDataWriter.write(powerEvaluator.getEvaluation());
                  rawDataWriter.newLine();
                }
                rawDataWriter.newLine();

                // process stats
                for (StatAnalysisProcessor statAnalysisProcessor : statAnalysisProcessors)
                {
                  statAnalysisProcessor.analyseStats(evaluatorMap);
                }
              }
            }
          }
        }
      }
      rawDataWriter.close();

      // write stats analysis
      for (StatAnalysisProcessor statAnalysisProcessor : statAnalysisProcessors)
      {
        statsAnalysisWriter.write(statAnalysisProcessor.getReport());
        statsAnalysisWriter.newLine();
        statsAnalysisWriter.newLine();
      }
      statsAnalysisWriter.close();
    }
    catch (IOException e)
    {
      System.out.println("I/O exception.");
    }
    finally
    {
      if (rawDataWriter != null)
      {
        try
        {
          rawDataWriter.close();
        }
        catch (IOException e)
        {
          System.out.println("I/O exception when closing rawDataWriter.");
        }
      }
      if (statsAnalysisWriter != null)
      {
        try
        {
          statsAnalysisWriter.close();
        }
        catch (IOException e)
        {
          System.out.println("I/O exception when closing statsAnalysisWriter.");
        }
      }
    }
  }

  /**
   * Initialise the {@link StatAnalysisProcessor} objects and seed them with
   * their {@link SingleStatAnalysis} objects.
   */
  private static void initStatProcessors()
  {
    OneOnOneExpectedDamageStatAnalysisProcessor basicAttackCarefulAttackStatAnalysisProcessor = new OneOnOneExpectedDamageStatAnalysisProcessor();
    OneOnOneExpectedDamageStatAnalysisProcessor basicAttackCarefulAttack2StatAnalysisProcessor = new OneOnOneExpectedDamageStatAnalysisProcessor();
    OneOnOneExpectedDamageStatAnalysisProcessor twinStrikeBasicAttackStatAnalysisProcessor = new OneOnOneExpectedDamageStatAnalysisProcessor();
    OneOnOneExpectedDamageStatAnalysisProcessor twinStrikeCarefulAttackStatAnalysisProcessor = new OneOnOneExpectedDamageStatAnalysisProcessor();
    OneOnOneExpectedDamageStatAnalysisProcessor twinStrikeReapingStrike1HandedStatAnalysisProcessor = new OneOnOneExpectedDamageStatAnalysisProcessor();
    OneOnOneExpectedDamageStatAnalysisProcessor twinStrikeReapingStrike2HandedStatAnalysisProcessor = new OneOnOneExpectedDamageStatAnalysisProcessor();
    OneOnOneExpectedDamageStatAnalysisProcessor twinStrikeAlternateReapingStrike1HandedStatAnalysisProcessor = new OneOnOneExpectedDamageStatAnalysisProcessor();
    OneOnOneExpectedDamageStatAnalysisProcessor twinStrikeAlternateReapingStrike2HandedStatAnalysisProcessor = new OneOnOneExpectedDamageStatAnalysisProcessor();
    OneOnOneExpectedDamageStatAnalysisProcessor twinStrikeAlternate2ReapingStrike1HandedStatAnalysisProcessor = new OneOnOneExpectedDamageStatAnalysisProcessor();
    OneOnOneExpectedDamageStatAnalysisProcessor twinStrikeAlternate2ReapingStrike2HandedStatAnalysisProcessor = new OneOnOneExpectedDamageStatAnalysisProcessor();

    basicAttackCarefulAttackStatAnalysisProcessor.setContestant1Class(BasicAttackEvaluator.class);
    basicAttackCarefulAttackStatAnalysisProcessor.setContestant2Class(CarefulAttackEvaluator.class);
    basicAttackCarefulAttackStatAnalysisProcessor
        .addStatAnalysis(new OneOnOneExpectedDamageSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
            "Basic Attack vs. Careful Attack/Sure Strike", basicAttackCarefulAttackStatAnalysisProcessor));
    basicAttackCarefulAttackStatAnalysisProcessor
        .addStatAnalysis(new OneOnOneDamageBonusAbilityModDeltaSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
            "Basic Attack vs. Careful Attack/Sure Strike Damage Bonus/Ability Mod Distribution",
            basicAttackCarefulAttackStatAnalysisProcessor));
    basicAttackCarefulAttackStatAnalysisProcessor
        .addStatAnalysis(new OneOnOneHitChanceDistributionSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
            "Basic Attack vs. Careful Attack/Sure Strike Hit Chance Distribution",
            basicAttackCarefulAttackStatAnalysisProcessor));
    statAnalysisProcessors.add(basicAttackCarefulAttackStatAnalysisProcessor);

    basicAttackCarefulAttack2StatAnalysisProcessor.setContestant1Class(BasicAttackEvaluator.class);
    basicAttackCarefulAttack2StatAnalysisProcessor.setContestant2Class(CarefulAttackEvaluator2.class);
    basicAttackCarefulAttack2StatAnalysisProcessor
    .addStatAnalysis(new OneOnOneExpectedDamageSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Basic Attack vs. Careful Attack/Sure Strike, +4 to hit", basicAttackCarefulAttack2StatAnalysisProcessor));
    basicAttackCarefulAttack2StatAnalysisProcessor
    .addStatAnalysis(new OneOnOneDamageBonusAbilityModDeltaSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Basic Attack vs. Careful Attack/Sure Strike, +4 to hit Damage Bonus/Ability Mod Distribution",
        basicAttackCarefulAttack2StatAnalysisProcessor));
    basicAttackCarefulAttack2StatAnalysisProcessor
    .addStatAnalysis(new OneOnOneHitChanceDistributionSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Basic Attack vs. Careful Attack/Sure Strike, +4 to hit Hit Chance Distribution",
        basicAttackCarefulAttack2StatAnalysisProcessor));
    statAnalysisProcessors.add(basicAttackCarefulAttack2StatAnalysisProcessor);

    twinStrikeCarefulAttackStatAnalysisProcessor.setContestant1Class(TwinStrikeEvaluator.class);
    twinStrikeCarefulAttackStatAnalysisProcessor.setContestant2Class(CarefulAttackEvaluator.class);
    twinStrikeCarefulAttackStatAnalysisProcessor
        .addStatAnalysis(new OneOnOneExpectedDamageSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
            "Twin Strike vs. Careful Attack", twinStrikeCarefulAttackStatAnalysisProcessor));
    twinStrikeCarefulAttackStatAnalysisProcessor
        .addStatAnalysis(new OneOnOneDamageBonusAbilityModDeltaSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
            "Twin Strike vs. Careful Attack Damage Bonus/Ability Mod Distribution",
            twinStrikeCarefulAttackStatAnalysisProcessor));
    statAnalysisProcessors.add(twinStrikeCarefulAttackStatAnalysisProcessor);

    twinStrikeBasicAttackStatAnalysisProcessor.setContestant1Class(TwinStrikeEvaluator.class);
    twinStrikeBasicAttackStatAnalysisProcessor.setContestant2Class(BasicAttackEvaluator.class);
    twinStrikeBasicAttackStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneExpectedDamageSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike vs. Basic Attack", twinStrikeBasicAttackStatAnalysisProcessor));
    twinStrikeBasicAttackStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneDamageBonusAbilityModDeltaSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike vs. Basic Attack Damage Bonus/Ability Mod Distribution",
        twinStrikeBasicAttackStatAnalysisProcessor));
    statAnalysisProcessors.add(twinStrikeBasicAttackStatAnalysisProcessor);

    twinStrikeReapingStrike1HandedStatAnalysisProcessor.setContestant1Class(TwinStrikeEvaluator.class);
    twinStrikeReapingStrike1HandedStatAnalysisProcessor.setContestant2Class(ReapingStrike1HandedEvaluator.class);
    twinStrikeReapingStrike1HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneExpectedDamageSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike vs. Reaping Strike (1-Handed)", twinStrikeReapingStrike1HandedStatAnalysisProcessor));
    twinStrikeReapingStrike1HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneDamageBonusAbilityModDeltaSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike vs. Reaping Strike (1-Handed) Damage Bonus/Ability Mod Distribution",
        twinStrikeReapingStrike1HandedStatAnalysisProcessor));
    twinStrikeReapingStrike1HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneHitChanceDistributionSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike vs. Reaping Strike (1-Handed) Hit Chance Distribution",
        twinStrikeReapingStrike1HandedStatAnalysisProcessor));
    statAnalysisProcessors.add(twinStrikeReapingStrike1HandedStatAnalysisProcessor);

    twinStrikeReapingStrike2HandedStatAnalysisProcessor.setContestant1Class(TwinStrikeEvaluator.class);
    twinStrikeReapingStrike2HandedStatAnalysisProcessor.setContestant2Class(ReapingStrike2HandedEvaluator.class);
    twinStrikeReapingStrike2HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneExpectedDamageSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike vs. Reaping Strike (2-Handed)", twinStrikeReapingStrike2HandedStatAnalysisProcessor));
    twinStrikeReapingStrike2HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneDamageBonusAbilityModDeltaSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike vs. Reaping Strike (2-Handed) Damage Bonus/Ability Mod Distribution",
        twinStrikeReapingStrike2HandedStatAnalysisProcessor));
    twinStrikeReapingStrike2HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneHitChanceDistributionSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike vs. Reaping Strike (2-Handed) Hit Chance Distribution",
        twinStrikeReapingStrike1HandedStatAnalysisProcessor));
    statAnalysisProcessors.add(twinStrikeReapingStrike2HandedStatAnalysisProcessor);

    twinStrikeAlternateReapingStrike1HandedStatAnalysisProcessor.setContestant1Class(TwinStrikeEvaluator2.class);
    twinStrikeAlternateReapingStrike1HandedStatAnalysisProcessor.setContestant2Class(ReapingStrike1HandedEvaluator.class);
    twinStrikeAlternateReapingStrike1HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneExpectedDamageSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -2 to hit vs. Reaping Strike (1-Handed)", twinStrikeAlternateReapingStrike1HandedStatAnalysisProcessor));
    twinStrikeAlternateReapingStrike1HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneDamageBonusAbilityModDeltaSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -2 to hit vs. Reaping Strike (1-Handed) Damage Bonus/Ability Mod Distribution",
        twinStrikeAlternateReapingStrike1HandedStatAnalysisProcessor));
    twinStrikeAlternateReapingStrike1HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneHitChanceDistributionSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -2 to hit vs. Reaping Strike (1-Handed) Hit Chance Distribution",
        twinStrikeAlternateReapingStrike1HandedStatAnalysisProcessor));
    statAnalysisProcessors.add(twinStrikeAlternateReapingStrike1HandedStatAnalysisProcessor);
    
    twinStrikeAlternateReapingStrike2HandedStatAnalysisProcessor.setContestant1Class(TwinStrikeEvaluator2.class);
    twinStrikeAlternateReapingStrike2HandedStatAnalysisProcessor.setContestant2Class(ReapingStrike2HandedEvaluator.class);
    twinStrikeAlternateReapingStrike2HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneExpectedDamageSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -2 to hit vs. Reaping Strike (2-Handed)", twinStrikeAlternateReapingStrike2HandedStatAnalysisProcessor));
    twinStrikeAlternateReapingStrike2HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneDamageBonusAbilityModDeltaSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -2 to hit vs. Reaping Strike (2-Handed) Damage Bonus/Ability Mod Distribution",
        twinStrikeAlternateReapingStrike2HandedStatAnalysisProcessor));
    twinStrikeAlternateReapingStrike2HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneHitChanceDistributionSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -2 to hit vs. Reaping Strike (2-Handed) Hit Chance Distribution",
        twinStrikeAlternateReapingStrike2HandedStatAnalysisProcessor));
    statAnalysisProcessors.add(twinStrikeAlternateReapingStrike2HandedStatAnalysisProcessor);

    twinStrikeAlternate2ReapingStrike1HandedStatAnalysisProcessor.setContestant1Class(TwinStrikeEvaluator3.class);
    twinStrikeAlternate2ReapingStrike1HandedStatAnalysisProcessor.setContestant2Class(ReapingStrike1HandedEvaluator.class);
    twinStrikeAlternate2ReapingStrike1HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneExpectedDamageSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -4 to hit vs. Reaping Strike (1-Handed)", twinStrikeAlternate2ReapingStrike1HandedStatAnalysisProcessor));
    twinStrikeAlternate2ReapingStrike1HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneDamageBonusAbilityModDeltaSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -4 to hit vs. Reaping Strike (1-Handed) Damage Bonus/Ability Mod Distribution",
        twinStrikeAlternate2ReapingStrike1HandedStatAnalysisProcessor));
    twinStrikeAlternate2ReapingStrike1HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneHitChanceDistributionSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -4 to hit vs. Reaping Strike (1-Handed) Hit Chance Distribution",
        twinStrikeAlternate2ReapingStrike1HandedStatAnalysisProcessor));
    statAnalysisProcessors.add(twinStrikeAlternate2ReapingStrike1HandedStatAnalysisProcessor);
    
    twinStrikeAlternate2ReapingStrike2HandedStatAnalysisProcessor.setContestant1Class(TwinStrikeEvaluator3.class);
    twinStrikeAlternate2ReapingStrike2HandedStatAnalysisProcessor.setContestant2Class(ReapingStrike2HandedEvaluator.class);
    twinStrikeAlternate2ReapingStrike2HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneExpectedDamageSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -4 to hit vs. Reaping Strike (2-Handed)", twinStrikeAlternate2ReapingStrike2HandedStatAnalysisProcessor));
    twinStrikeAlternate2ReapingStrike2HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneDamageBonusAbilityModDeltaSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -4 to hit vs. Reaping Strike (2-Handed) Damage Bonus/Ability Mod Distribution",
        twinStrikeAlternate2ReapingStrike2HandedStatAnalysisProcessor));
    twinStrikeAlternate2ReapingStrike2HandedStatAnalysisProcessor
    .addStatAnalysis(new OneOnOneHitChanceDistributionSingleStatAnalysis<OneOnOneExpectedDamageStatAnalysisProcessor>(
        "Twin Strike, -4 to hit vs. Reaping Strike (2-Handed) Hit Chance Distribution",
        twinStrikeAlternate2ReapingStrike2HandedStatAnalysisProcessor));
    statAnalysisProcessors.add(twinStrikeAlternate2ReapingStrike2HandedStatAnalysisProcessor);
  }
}
