package powerEvaluators;

/**
 * Evaluates a weapon-based power.
 * 
 * @param <T> The subclass of the implementing {@link PowerEvaluator}.
 * @param <S> The type of the evaluation's return value.
 */
public abstract class PowerEvaluator<T extends PowerEvaluator<T, S>, S> implements Comparable<T>
{
  /**
   * The die used for this weapon's damage roll. e.g. 8 (longsword), 10 (bastard
   * sword)
   */
  private int weaponDamageDie;
  /**
   * The average damage for this weapon. This value is auto-calculated.
   */
  private double averageDamage;
  /**
   * The critical range of the attack, expressed as an integer. e.g. 20, 19, 18
   */
  private int critRange;
  /**
   * The probability that a given attack will hit. This takes into account all
   * modifiers to the attack roll, but doesn't include modifiers applied by the
   * power being evaluated (for instance, the +2 provided by the Ranger's
   * Careful Attack at-will power). This number varies between 5% and 95%
   * (taking natural 1s (auto-miss) and 20s (auto-hit) into account).
   */
  private int rollRequiredToHit;
  /**
   * The bonus to the attack's damage roll. This DOES NOT include the relevant
   * ability score mod or any bonus damage applied by the power being evaluated.
   */
  private int bonusDamage;
  /**
   * The ability score mod that is normally applied to the damage roll for this
   * sort of attack.
   */
  private int abilityModBonus;

  /**
   * Constructor.
   */
  public PowerEvaluator()
  {
    super();
  }

  /**
   * 
   * @return A label describing this power.
   */
  public abstract String getName();

  /**
   * 
   * @return A string summarising the evaluation of this power.
   */
  public abstract String getEvaluation();
  
  /**
   * 
   * @return The result of the evaluation.
   */
  public abstract S evaluateImpl();

  /**
   * Sets the variables for the expected value calculation, then runs the
   * calculation using {@link #calculateExpectedDamage()}.
   * 
   * @param weaponDamageDie
   *          {@link #weaponDamageDie}
   * @param critRange
   *          {@link #critRange}
   * @param probabilityToHit
   *          {@link #rollRequiredToHit}
   * @param bonusDamage
   *          {@link #bonusDamage}
   * @param abilityModbonus
   *          {@link #abilityModBonus}
   * @return The expected damage for this attack, given the supplied parameters.
   */
  public S evaluate(int weaponDamageDie, int critRange, int probabilityToHit, int bonusDamage, int abilityModbonus)
  {
    this.weaponDamageDie = weaponDamageDie;
    averageDamage = (weaponDamageDie + 1.0) / 2;
    this.critRange = critRange;
    this.rollRequiredToHit = probabilityToHit;
    this.bonusDamage = bonusDamage;
    this.abilityModBonus = abilityModbonus;
    
    return evaluateImpl();
  }

  /**
   * The probability of an attack being a critical hit.
   * 
   * @return A probability between 0 and 1.
   */
  public double getProbabilityOfCrit()
  {
    return (critRange > rollRequiredToHit) ? ((21 - critRange) * 0.05) : ((21 - rollRequiredToHit) * 0.05);
  }

  /**
   * The probability of an attack being a hit, but not a critical hit.
   * 
   * @return A probability between 0 and 1.
   */
  public double getProbabilityOfHit()
  {
    return (critRange > rollRequiredToHit) ? ((critRange - rollRequiredToHit) * 0.05) : 0;
  }

  /**
   * The probability of an attack being a miss.
   * 
   * @return A probability between 0 and 1.
   */
  public double getProbabilityOfMiss()
  {
    return (critRange > rollRequiredToHit) ? ((rollRequiredToHit - 1) * 0.05) : ((critRange - 1) * 0.05);
  }

  /**
   * 
   * @return The die type for the weapon used.
   */
  protected int getWeaponDamageDie()
  {
    return weaponDamageDie;
  }

  /**
   * 
   * @return The average roll for a weapon of this die type.
   */
  protected double getAverageDamage()
  {
    return averageDamage;
  }

  /**
   * 
   * @return The critical range for the weapon.
   */
  int getCritRange()
  {
    return critRange;
  }

  /**
   * The probability that the attack will hit, not including modifiers applied
   * by the power being evaluated.
   * 
   * @return A probability between 0 and 1.
   */
  public int getRollRequiredToHit()
  {
    return rollRequiredToHit;
  }

  /**
   * 
   * @return The bonus damage applied to this attack, not including the ability
   *         score mod or bonus damage applied by the power being evaluated.
   */
  public int getBonusDamage()
  {
    return bonusDamage;
  }

  /**
   * 
   * @return The ability score mod applied to the weapon's damage when using the
   *         power being evaluated.
   */
  public int getAbilityModBonus()
  {
    return abilityModBonus;
  }

  /**
   * 
   * @param weaponDamageDie {@link #weaponDamageDie}
   */
  public void setWeaponDamageDie(int weaponDamageDie)
  {
    this.weaponDamageDie = weaponDamageDie;
  }

  /**
   * 
   * @param critRange {@link #critRange}
   */
  public void setCritRange(int critRange)
  {
    this.critRange = critRange;
  }

  /**
   * 
   * @param rollRequiredToHit {@link #rollRequiredToHit}
   */
  public void setRollRequiredToHit(int rollRequiredToHit)
  {
    this.rollRequiredToHit = rollRequiredToHit;
    if (this.rollRequiredToHit > 20)
    {
      this.rollRequiredToHit = 20;
    }
    else if (this.rollRequiredToHit < 2)
    {
      this.rollRequiredToHit = 2;
    }
  }

  /**
   * 
   * @param bonusDamage {@link #bonusDamage}
   */
  public void setBonusDamage(int bonusDamage)
  {
    this.bonusDamage = bonusDamage;
  }

  /**
   * 
   * @param abilityModBonus {@link #abilityModBonus}
   */
  public void setAbilityModBonus(int abilityModBonus)
  {
    this.abilityModBonus = abilityModBonus;
  }
}