package powerEvaluators;

/**
 * Superclass for power evaluators. Provides the means to calculate the expected
 * damage of an attack based on a number of factors, including the crit range of
 * the weapon, the base damage of the weapon, the ability score mod of the
 * attacker, etc.
 * 
 * The expected damage is the mathematical expectation of the damage for a given
 * attack. See the link below for an explanation of mathematical expectation and
 * its role in probability theory:
 * 
 * @see http://en.wikipedia.org/wiki/Expected_value
 */
public abstract class PowerExpectedDamageEvaluator extends
    PowerEvaluator<PowerExpectedDamageEvaluator, Double>
{
  /**
   * The expected damage calculated by the
   * {@link #evaluate(int, int, int, int, int)}/{@link #calculateExpectedDamage()}
   * methods.
   */
  private double expectedDamage;

  /**
   * 
   * @return The expected damage of the power represented by the implementing
   *         {@link PowerExpectedDamageEvaluator}.
   */
  public abstract Double calculateExpectedDamage();

  @Override
  public Double evaluateImpl()
  {
    expectedDamage = calculateExpectedDamage();
    
    return expectedDamage;
  }

  @Override
  public int compareTo(PowerExpectedDamageEvaluator o)
  {
    return Double.compare(this.expectedDamage, o.expectedDamage);
  }

  @Override
  public String getEvaluation()
  {
    StringBuilder evaluation = new StringBuilder();

    evaluation.append(getName());
    evaluation.append(": expected damage: ");
    evaluation.append(expectedDamage);

    return evaluation.toString();
  }

  /**
   * 
   * @return {@link #expectedDamage}
   */
  public double getExpectedDamage()
  {
    return expectedDamage;
  }
}
