package statAnalysis;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import powerEvaluators.PowerExpectedDamageEvaluator;
import statAnalysis.singleStatAnalysis.SingleStatAnalysis;

/**
 * 
 * Superclass for all classes which track the number of occurrences of a
 * particular statistical event.
 */
public abstract class StatAnalysisProcessor
{
  /**
   * The number of stats this {@link StatAnalysisProcessor} has processed.
   */
  private int numberOfStatsAnalysed = 0;
  /**
   * A list of the {@link SingleStatAnalysis} objects which will analyse and
   * provide reports on a particular property of the event being evaluated.
   */
  private List<SingleStatAnalysis<? extends StatAnalysisProcessor>> statAnalysisList = new ArrayList<SingleStatAnalysis<? extends StatAnalysisProcessor>>();
  /**
   * Provides a context for {@link SingleStatAnalysis} objects to store state
   * information across the lifespan of the {@link StatAnalysisProcessor}.
   */
  private Map<Class<? extends SingleStatAnalysis<? extends StatAnalysisProcessor>>, Map<String, Object>> statAnalysisProcessorContext = new HashMap<Class<? extends SingleStatAnalysis<? extends StatAnalysisProcessor>>, Map<String, Object>>();

  /**
   * Constructor.
   */
  public StatAnalysisProcessor()
  {
  }

  /**
   * Constructor.
   * 
   * @param label
   *          This {@link StatAnalysisProcessor}'s {@link #label}.
   */
  public StatAnalysisProcessor(List<SingleStatAnalysis<? extends StatAnalysisProcessor>> statAnalysisList)
  {
    this.statAnalysisList = statAnalysisList;
  }
  
  /**
   * Add a {@link SingleStatAnalysis} object to this processor.
   * @param statAnalysis The {@link SingleStatAnalysis} to be added.
   */
  public void addStatAnalysis(SingleStatAnalysis<? extends StatAnalysisProcessor> statAnalysis)
  {
    statAnalysisList.add(statAnalysis);
  }

  /**
   * 
   * @return A string containing a report of this {@link StatAnalysisProcessor}'s
   *         analysis.
   */
  public String getReport()
  {
    StringBuilder report = new StringBuilder();

    for (SingleStatAnalysis<? extends StatAnalysisProcessor> singleStatAnalysis : statAnalysisList)
    {
      report.append(singleStatAnalysis.report());
      report.append("\n");
    }

    return report.toString();
  }

  /**
   * Calls {@link #recordStatImpl(Map)} and increments the number of stats
   * analysed counter.
   * 
   * @param statsMap
   *          The list of {@link PowerExpectedDamageEvaluator} objects
   *          representing a single evaluation state.
   */
  public void analyseStats(
      Map<Class, ? extends PowerExpectedDamageEvaluator> statsMap)
  {
    numberOfStatsAnalysed++;
    for (SingleStatAnalysis<? extends StatAnalysisProcessor> singleStatAnalysis : statAnalysisList)
    {
      singleStatAnalysis.preAnalysis(statsMap);
    }
    for (SingleStatAnalysis<? extends StatAnalysisProcessor> singleStatAnalysis : statAnalysisList)
    {
      singleStatAnalysis.analysis(statsMap);
    }
    for (SingleStatAnalysis<? extends StatAnalysisProcessor> singleStatAnalysis : statAnalysisList)
    {
      singleStatAnalysis.postAnalysis(statsMap);
    }
  }

  /**
   * 
   * @return {@link #numberOfStatsAnalysed}
   */
  public int getNumberOfStatsAnalysed()
  {
    return numberOfStatsAnalysed;
  }

  /**
   * Sets the specified value in this processor's context.
   * 
   * @param key
   *          The key under which the value is to be stored.
   * @param value
   *          The value to be stored.
   */
  public void setAttribute(Class singleStatAnalysisClass, String key, Object value)
  {
    Map<String, Object> singleStatAnalysisAttributes = statAnalysisProcessorContext.get(singleStatAnalysisClass);
    
    if (singleStatAnalysisAttributes == null)
    {
      singleStatAnalysisAttributes = new HashMap<String, Object>();
      statAnalysisProcessorContext.put(singleStatAnalysisClass, singleStatAnalysisAttributes);
    }
    singleStatAnalysisAttributes.put(key, value);
  }

  /**
   * 
   * @param key
   *          The key under which the value is to be stored.
   * @return The value in this processor's context corresponding to the
   *         specified key.
   */
  public Object getAttribute(Class singleStatAnalysisClass, String key)
  {
    Object attribute = null;
    Map<String, Object> singleStatAnalysisAttributes = statAnalysisProcessorContext.get(singleStatAnalysisClass);

    if (singleStatAnalysisAttributes == null)
    {
      System.out.println(getClass() + ".getAttribute(): No attributes stored under SingleStatAnalysis class: " + singleStatAnalysisClass + ".");
    }
    else
    {
      attribute = singleStatAnalysisAttributes.get(key);
    }
    
    return attribute;
  }
}
