package statAnalysis.singleStatAnalysis;

import java.util.Map;

import powerEvaluators.PowerExpectedDamageEvaluator;
import statAnalysis.OneOnOneStatAnalysisProcessor;
import statAnalysis.StatAnalysisProcessor;

/**
 * Reports on the average expected damage in wins when two powers are matched
 * head-to-head.
 * 
 * @param <T>
 *          The type of the {@link OneOnOneStatAnalysisProcessor} managing this
 *          single stat analysis object.
 */
public class OneOnOneExpectedDamageSingleStatAnalysis<T extends OneOnOneStatAnalysisProcessor> extends
    OneOnOneSingleStatAnalysis<T>
{
  /**
   * The key used to store the total expected damage in
   * {@link StatAnalysisProcessor#setAttribute(Class, String, Object)}. The
   * total expected damage is the sum of all expected damages values for all
   * 'win' scenarios. This value is used to calculate the average expected
   * damage of a 'win' scenario.
   */
  private static final String TOTAL_EXPECTED_DAMAGE_ATTRIBUTE = "totalExpectedDamage";

  /**
   * Constructor.
   * 
   * @param statAnalysisProcessor
   *          {@link #statAnalysisProcessor}
   */
  public OneOnOneExpectedDamageSingleStatAnalysis(String label, T statAnalysisProcessor)
  {
    super(label, statAnalysisProcessor);
  }

  @Override
  public String report()
  {
    StringBuilder report = new StringBuilder(super.report());
    Double totalExpectedDamage = (Double)getStatAnalysisProcessor().getAttribute(OneOnOneSingleStatAnalysis.class,
        TOTAL_EXPECTED_DAMAGE_ATTRIBUTE);

    report.append("Average damage in win scenarios: ");
    report.append(getAverageExpectedDamage(totalExpectedDamage));
    report.append("\n");

    return report.toString();
  }

  @Override
  public void analysis(
      Map<Class, ? extends PowerExpectedDamageEvaluator> statsMap)
  {
    double expectedDamageDelta = statsMap.get(getStatAnalysisProcessor().getContestant1Class()).getExpectedDamage()
        - statsMap.get(getStatAnalysisProcessor().getContestant2Class()).getExpectedDamage();

    if (expectedDamageDelta >= 0)
    {
      Double totalExpectedDamage = (Double)getStatAnalysisProcessor().getAttribute(OneOnOneSingleStatAnalysis.class,
          TOTAL_EXPECTED_DAMAGE_ATTRIBUTE);

      if (totalExpectedDamage == null)
      {
        totalExpectedDamage = new Double(0);
      }
      totalExpectedDamage += expectedDamageDelta;
      getStatAnalysisProcessor().setAttribute(OneOnOneSingleStatAnalysis.class, TOTAL_EXPECTED_DAMAGE_ATTRIBUTE,
          totalExpectedDamage);
    }
  }

  /**
   * @param totalExpectedDamage
   *          The sum of the expected damage values from all evaluated win
   *          scenarios.
   * @return The average expected damage of 'win' scenarios.
   */
  public double getAverageExpectedDamage(Double totalExpectedDamage)
  {
    return totalExpectedDamage / getStatAnalysisProcessor().getWinCounter();
  }
}
