package statAnalysis.singleStatAnalysis;

import java.util.Map;
import java.util.TreeMap;

import powerEvaluators.PowerEvaluator;
import powerEvaluators.PowerExpectedDamageEvaluator;
import statAnalysis.OneOnOneStatAnalysisProcessor;

/**
 * Reports on the average expected damage in wins when two powers are matched head-to-head.
 * @param <T>
 *          The type of the {@link OneOnOneStatAnalysisProcessor} managing this
 *          single stat analysis object.
 *          @param <S> The key for the win distribution map.
 */
abstract class OneOnOneWinDistributionSingleStatAnalysis<T extends OneOnOneStatAnalysisProcessor, S>
    extends OneOnOneSingleStatAnalysis<T>
{
  /**
   * Tracks the distribution of the delta between ability score mod and damage
   * bonus. The key is the delta itself while the value is a
   * {@link DistributionHelper} object, which tracks the
   * occurrence of both 'win' and 'loss' scenarios at that particular delta.
   */
  private Map<S, DistributionHelper> winDistributionMap = new TreeMap<S, DistributionHelper>();

  /**
   * Constructor.
   * 
   * @param statAnalysisProcessor
   *          {@link #statAnalysisProcessor}
   */
  public OneOnOneWinDistributionSingleStatAnalysis(String label, T statAnalysisProcessor)
  {
    super(label, statAnalysisProcessor);
  }
  
  /**
   * 
   * @param statsMap
   *          The list of {@link PowerExpectedDamageEvaluator} objects
   *          representing a single evaluation state.
   * @return The key representing the evaluation data in this statsMap.
   */
  protected abstract S getKey(Map<Class, ? extends PowerExpectedDamageEvaluator> statsMap);
  
  @Override
  public void analysis(Map<Class, ? extends PowerExpectedDamageEvaluator> statsMap)
  {
    PowerEvaluator<? extends PowerEvaluator<?, ?>, ?> powerEvaluator = statsMap.get(getStatAnalysisProcessor().getContestant1Class());
    
    if (getStatAnalysisProcessor().isAWin(statsMap))
    {
      addWin(getKey(statsMap));
    }
    else
    {
      addLoss(getKey(statsMap));
    }
  }

  /**
   * Tracks the number of losses associated with a particular point in the distribution.
   * 
   * @param key
   *          The key in the distribution map for a
   *          particular 'win' scenario.
   */
  protected void addWin(S key)
  {
    DistributionHelper damageBonusAbilityModDeltaCount = winDistributionMap
        .get(key);

    if (damageBonusAbilityModDeltaCount == null)
    {
      DistributionHelper deltaWrapper = new DistributionHelper();

      deltaWrapper.incrementWinCount();
      winDistributionMap.put(key, deltaWrapper);
    }
    else
    {
      DistributionHelper distributionHelper = winDistributionMap
          .get(key);

      distributionHelper.incrementWinCount();
    }
  }

  /**
   * Tracks the number of losses associated with a particular point in the distribution.
   * 
   * @param key
   *          The key in the distribution map for a
   *          particular 'loss' scenario.
   */
  protected void addLoss(S key)
  {
    DistributionHelper damageModAbilityModDeltaCount = winDistributionMap
        .get(key);

    if (damageModAbilityModDeltaCount == null)
    {
      DistributionHelper distributionHelper = new DistributionHelper();

      distributionHelper.incrementLossCount();
      winDistributionMap.put(key, distributionHelper);
    }
    else
    {
      DistributionHelper distributionHelper = winDistributionMap
          .get(key);

      distributionHelper.incrementLossCount();
    }
  }
  
  /**
   * Helper class which tracks the number of wins and losses for a particular
   * key in the distribution map.
   * 
   */
  protected class DistributionHelper
  {
    private int winCount;
    private int lossCount;

    protected int getWinCount()
    {
      return winCount;
    }

    protected void incrementWinCount()
    {
      winCount++;
    }

    protected int getLossCount()
    {
      return lossCount;
    }

    protected void incrementLossCount()
    {
      lossCount++;
    }
  }

  /**
   * 
   * @return {@link #winDistributionMap}
   */
  protected Map<S, DistributionHelper> getWinDistributionMap()
  {
    return winDistributionMap;
  }
}