package statAnalysis.singleStatAnalysis;

import java.util.Map;

import powerEvaluators.PowerExpectedDamageEvaluator;
import statAnalysis.StatAnalysisProcessor;

/**
 * Analyses a single aspect of a list of {@link PowerExpectedDamageEvaluator}
 * objects.
 * 
 * @param T
 *          The {@link StatAnalysisProcessor} managing this stat analysis.
 */
public abstract class SingleStatAnalysis<T extends StatAnalysisProcessor>
{
  /**
   * The {@link StatAnalysisProcessor} managing this stat analysis.
   */
  private T statAnalysisProcessor;
  /**
   * A text label describing the statistical analysis represented by this
   * {@link StatAnalysisProcessor}.
   */
  private String label;

  /**
   * 
   * @return A report of the analysis of this {@link SingleStatAnalysis}'s
   *         stat.
   */
  public abstract String report();

  /**
   * Use the supplied {@link PowerExpectedDamageEvaluator} objects to obtain the
   * expected damage for the powers you are analysing. Use these expected damage
   * values to perform your analysis.
   * 
   * @param statsMap
   *          A {@link Map} containing a list of
   *          {@link PowerExpectedDamageEvaluator} objects, keyed on the
   *          {@link Class} of those objects.
   */
  public abstract void analysis(Map<Class, ? extends PowerExpectedDamageEvaluator> statsMap);

  /**
   * Hook for handling any pre-analysis functionality. This method runs before
   * the main {@link #analysis(Map)} method.
   * 
   * @param statsMap
   *          A map of {@link PowerExpectedDamageEvaluator} objects, keyed on
   *          the class of that evaluator.
   */
  public void preAnalysis(Map<Class, ? extends PowerExpectedDamageEvaluator> statsMap)
  {
  }

  /**
   * Hook for handling any post-analysis functionality. This method runs after
   * the main {@link #analysis(Map)} method.
   * 
   * @param statsMap
   *          A map of {@link PowerExpectedDamageEvaluator} objects, keyed on
   *          the class of that evaluator.
   */
  public void postAnalysis(Map<Class, ? extends PowerExpectedDamageEvaluator> statsMap)
  {
  }

  /**
   * Constructor.
   * 
   * @param statAnalysisProcessor
   *          {@link #statAnalysisProcessor}
   */
  SingleStatAnalysis(String label, T statAnalysisProcessor)
  {
    this.statAnalysisProcessor = statAnalysisProcessor;
    this.label = label;
  }

  /**
   * 
   * @param countedValue
   *          The value to be converted.
   * @return The first value passed as a percentage of the second value passed.
   */
  double convertCountedValueToPercentage(int countedValue, int totalCount)
  {
    return (countedValue * 100.0) / totalCount;
  }

  /**
   * 
   * @return {@link #statAnalysisProcessor}
   */
  protected T getStatAnalysisProcessor()
  {
    return statAnalysisProcessor;
  }

  /**
   * 
   * @param statAnalysisProcessor {@link #statAnalysisProcessor}
   */
  protected void setStatAnalysisProcessor(T statAnalysisProcessor)
  {
    this.statAnalysisProcessor = statAnalysisProcessor;
  }

  /**
   * 
   * @return {@link #label}
   */
  String getLabel()
  {
    return label;
  }
}
